@@ -1,10 +1,10 @@
 package Net::DNS::ZoneFile;
 
 #
-# $Id: ZoneFile.pm 1224 2014-07-01 07:57:42Z willem $
+# $Id: ZoneFile.pm 1277 2014-10-20 07:46:37Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1224 $)[1];
+$VERSION = (qw$LastChangedRevision: 1277 $)[1];
 
 
 =head1 NAME
@@ -447,7 +447,7 @@ sub _getline {				## get line from current source
 		next unless /\S/;				# discard blank line
 		next if /^\s*;/;				# discard comment line
 
-		if (/\(/) {					# concatenate multi-line RR
+		if (/["(]/) {					# concatenate multi-line RR
 			s/\\\\/\\092/g;				# disguise escaped escape
 			s/\\"/\\034/g;				# disguise escaped quote
 			s/\\\(/\\040/g;				# disguise escaped bracket
@@ -466,6 +466,19 @@ sub _getline {				## get line from current source
 					last if $token[$#token] eq ')';
 				}
 				$_ = join ' ', @token;		# reconstitute RR string
+
+			} elsif ( $token[$#token] =~ /^"[^"]+$/ ) {
+				while (<$fh>) {
+					s/\\\\/\\092/g;		# disguise escaped escape
+					s/\\"/\\034/g;		# disguise escaped quote
+					s/\\\(/\\040/g;		# disguise escaped bracket
+					s/\\\)/\\041/g;		# disguise escaped bracket
+					s/\\;/\\059/g;		# disguise escaped semicolon
+					substr( $_, 0, 0 ) = pop @token || '';	  # splice multi-line string
+					push @token, grep defined && length, split /$LEX_REGEX/o;
+					last unless $token[$#token] =~ /^"[^"]+$/;
+				}
+				$_ = join ' ', @token;		# reconstitute RR string
 			}
 		}
 
@@ -535,7 +548,7 @@ sub _include {				## open $INCLUDE file
 	my $file = _filename(shift);
 	my $root = shift;
 
-	my @discipline = ( join ':', '<', PerlIO::get_layers $self->{handle} ) if PERLIO;
+	my @discipline = PERLIO ? ( join ':', '<', PerlIO::get_layers $self->{handle} ) : ();
 	my $handle = new FileHandle( $file, @discipline ) or croak qq(open: "$file" $!);
 
 	delete $self->{latest};					# forbid empty owner field
